@@ -1,9 +1,11 @@
 import asyncio
+import hashlib
 import logging
-import operator
 import typing as t
 from collections import defaultdict
+from itertools import starmap
 
+from crackerjack.services.cache import CrackerjackCache
 from crackerjack.services.debug import get_ai_agent_debugger
 
 from .base import (
@@ -11,15 +13,34 @@ from .base import (
     FixResult,
     Issue,
     IssueType,
-    Priority,
     SubAgent,
     agent_registry,
 )
 from .tracker import get_agent_tracker
 
+# Static mapping for O(1) agent lookup by issue type
+ISSUE_TYPE_TO_AGENTS = {
+    IssueType.COMPLEXITY: ["RefactoringAgent"],
+    IssueType.DEAD_CODE: ["RefactoringAgent", "ImportOptimizationAgent"],
+    IssueType.SECURITY: ["SecurityAgent"],
+    IssueType.PERFORMANCE: ["PerformanceAgent", "RefactoringAgent"],
+    IssueType.TEST_FAILURE: ["TestCreationAgent", "TestSpecialistAgent"],
+    IssueType.TYPE_ERROR: ["TestCreationAgent", "RefactoringAgent"],
+    IssueType.FORMATTING: ["FormattingAgent", "ImportOptimizationAgent"],
+    IssueType.DOCUMENTATION: ["DocumentationAgent"],
+    IssueType.DRY_VIOLATION: ["DRYAgent", "RefactoringAgent"],
+    IssueType.IMPORT_ERROR: ["ImportOptimizationAgent", "RefactoringAgent"],
+    IssueType.COVERAGE_IMPROVEMENT: ["TestCreationAgent", "TestSpecialistAgent"],
+    IssueType.TEST_ORGANIZATION: ["TestSpecialistAgent", "TestCreationAgent"],
+    IssueType.DEPENDENCY: ["RefactoringAgent"],
+    IssueType.REGEX_VALIDATION: ["SecurityAgent", "RefactoringAgent"],
+}
+
 
 class AgentCoordinator:
-    def __init__(self, context: AgentContext) -> None:
+    def __init__(
+        self, context: AgentContext, cache: CrackerjackCache | None = None
+    ) -> None:
         self.context = context
         self.agents: list[SubAgent] = []
         self.logger = logging.getLogger(__name__)
@@ -27,7 +48,8 @@ class AgentCoordinator:
         self._collaboration_threshold = 0.7
         self.tracker = get_agent_tracker()
         self.debugger = get_ai_agent_debugger()
-        self.proactive_mode = True  # Enabled by default
+        self.proactive_mode = True
+        self.cache = cache or CrackerjackCache()
 
     def initialize_agents(self) -> None:
         self.agents = agent_registry.create_all(self.context)
@@ -54,49 +76,26 @@ class AgentCoordinator:
 
         issues_by_type = self._group_issues_by_type(issues)
 
-        overall_result = FixResult(success=True, confidence=1.0)
+        # Optimization: Run ALL issue types in parallel instead of sequential
+        tasks = list[t.Any](
+            starmap(self._handle_issues_by_type, issues_by_type.items())
+        )
 
-        for issue_type, type_issues in issues_by_type.items():
-            type_result = await self._handle_issues_by_type(issue_type, type_issues)
-            overall_result = overall_result.merge_with(type_result)
+        results = await asyncio.gather(*tasks, return_exceptions=True)
+
+        overall_result = FixResult(success=True, confidence=1.0)
+        for result in results:
+            if isinstance(result, FixResult):
+                overall_result = overall_result.merge_with(result)
+            else:
+                self.logger.error(f"Issue type handling failed: {result}")
+                overall_result.success = False
+                overall_result.remaining_issues.append(
+                    f"Type handling failed: {result}"
+                )
 
         return overall_result
 
-    async def handle_single_issue(self, issue: Issue) -> FixResult:
-        if not self.agents:
-            self.initialize_agents()
-
-        cache_key = issue.context_key
-        if cache_key in self._issue_cache:
-            cached_result = self._issue_cache[cache_key]
-            self.logger.info(f"Using cached result for {cache_key}")
-            self.tracker.track_cache_hit()
-            return cached_result
-
-        self.tracker.track_cache_miss()
-
-        agent_scores = await self._evaluate_agents_for_issue(issue)
-
-        if not agent_scores:
-            self.logger.warning(f"No agents can handle issue: {issue.message}")
-            return FixResult(
-                success=False,
-                confidence=0.0,
-                remaining_issues=[f"No agent available for: {issue.message}"],
-                recommendations=["Manual intervention required"],
-            )
-
-        best_agent, best_score = agent_scores[0]
-
-        if best_score >= self._collaboration_threshold:
-            result = await self._handle_with_single_agent(best_agent, issue)
-        else:
-            result = await self._handle_with_collaboration(agent_scores[:3], issue)
-
-        self._issue_cache[cache_key] = result
-
-        return result
-
     async def _handle_issues_by_type(
         self,
         issue_type: IssueType,
@@ -104,10 +103,25 @@ class AgentCoordinator:
     ) -> FixResult:
         self.logger.info(f"Handling {len(issues)} {issue_type.value} issues")
 
+        # Fast agent lookup using static mapping
+        preferred_agent_names = ISSUE_TYPE_TO_AGENTS.get(issue_type, [])
+        specialist_agents = []
+
+        # First, try to use agents from static mapping for O(1) lookup
         specialist_agents = [
-            agent for agent in self.agents if issue_type in agent.get_supported_types()
+            agent
+            for agent in self.agents
+            if agent.__class__.__name__ in preferred_agent_names
         ]
 
+        # Fallback: use traditional dynamic lookup if no static match
+        if not specialist_agents:
+            specialist_agents = [
+                agent
+                for agent in self.agents
+                if issue_type in agent.get_supported_types()
+            ]
+
         if not specialist_agents:
             self.logger.warning(f"No specialist agents for {issue_type.value}")
             return FixResult(
@@ -139,42 +153,117 @@ class AgentCoordinator:
 
         return combined_result
 
-    async def _evaluate_agents_for_issue(
-        self,
-        issue: Issue,
-    ) -> list[tuple[SubAgent, float]]:
-        evaluations: list[tuple[SubAgent, float]] = []
-
-        for agent in self.agents:
-            try:
-                confidence = await agent.can_handle(issue)
-                if confidence > 0.0:
-                    evaluations.append((agent, confidence))
-            except Exception as e:
-                self.logger.exception(f"Error evaluating {agent.name} for issue: {e}")
-
-        evaluations.sort(key=operator.itemgetter(1), reverse=True)
-        return evaluations
-
     async def _find_best_specialist(
         self,
         specialists: list[SubAgent],
         issue: Issue,
     ) -> SubAgent | None:
-        best_agent = None
-        best_score = 0.0
+        candidates = await self._score_all_specialists(specialists, issue)
+        if not candidates:
+            return None
+
+        best_agent, best_score = self._find_highest_scoring_agent(candidates)
+        return self._apply_built_in_preference(candidates, best_agent, best_score)
+
+    async def _score_all_specialists(
+        self, specialists: list[SubAgent], issue: Issue
+    ) -> list[tuple[SubAgent, float]]:
+        """Score all specialist agents for handling an issue."""
+        candidates: list[tuple[SubAgent, float]] = []
 
         for agent in specialists:
             try:
                 score = await agent.can_handle(issue)
-                if score > best_score:
-                    best_score = score
-                    best_agent = agent
+                candidates.append((agent, score))
             except Exception as e:
                 self.logger.exception(f"Error evaluating specialist {agent.name}: {e}")
 
+        return candidates
+
+    def _find_highest_scoring_agent(
+        self, candidates: list[tuple[SubAgent, float]]
+    ) -> tuple[SubAgent | None, float]:
+        """Find the agent with the highest score."""
+        best_agent = None
+        best_score = 0.0
+
+        for agent, score in candidates:
+            if score > best_score:
+                best_score = score
+                best_agent = agent
+
+        return best_agent, best_score
+
+    def _apply_built_in_preference(
+        self,
+        candidates: list[tuple[SubAgent, float]],
+        best_agent: SubAgent | None,
+        best_score: float,
+    ) -> SubAgent | None:
+        """Apply preference for built-in agents when scores are close."""
+        if not best_agent or best_score <= 0:
+            return best_agent
+
+        # Threshold for considering scores "close" (5% difference)
+        CLOSE_SCORE_THRESHOLD = 0.05
+
+        for agent, score in candidates:
+            if self._should_prefer_built_in_agent(
+                agent, best_agent, score, best_score, CLOSE_SCORE_THRESHOLD
+            ):
+                self._log_built_in_preference(
+                    agent, score, best_agent, best_score, best_score - score
+                )
+                return agent
+
         return best_agent
 
+    def _should_prefer_built_in_agent(
+        self,
+        agent: SubAgent,
+        best_agent: SubAgent,
+        score: float,
+        best_score: float,
+        threshold: float,
+    ) -> bool:
+        """Check if a built-in agent should be preferred over the current best."""
+        return (
+            agent != best_agent
+            and self._is_built_in_agent(agent)
+            and 0 < (best_score - score) <= threshold
+        )
+
+    def _log_built_in_preference(
+        self,
+        agent: SubAgent,
+        score: float,
+        best_agent: SubAgent,
+        best_score: float,
+        score_difference: float,
+    ) -> None:
+        """Log when preferring a built-in agent."""
+        self.logger.info(
+            f"Preferring built-in agent {agent.name} (score: {score:.2f}) "
+            f"over {best_agent.name} (score: {best_score:.2f}) "
+            f"due to {score_difference:.2f} threshold preference"
+        )
+
+    def _is_built_in_agent(self, agent: SubAgent) -> bool:
+        """Check if agent is a built-in Crackerjack agent."""
+        built_in_agent_names = {
+            "ArchitectAgent",
+            "DocumentationAgent",
+            "DRYAgent",
+            "FormattingAgent",
+            "ImportOptimizationAgent",
+            "PerformanceAgent",
+            "RefactoringAgent",
+            "SecurityAgent",
+            "TestCreationAgent",
+            "TestSpecialistAgent",
+        }
+        return agent.__class__.__name__ in built_in_agent_names
+
     async def _handle_with_single_agent(
         self,
         agent: SubAgent,
@@ -182,6 +271,16 @@ class AgentCoordinator:
     ) -> FixResult:
         self.logger.info(f"Handling issue with {agent.name}: {issue.message[:100]}")
 
+        # Create cache key from issue content
+        issue_hash = self._create_issue_hash(issue)
+
+        # Check cache for previous agent decision
+        cached_decision = self.cache.get_agent_decision(agent.name, issue_hash)
+        if cached_decision:
+            self.logger.debug(f"Using cached decision for {agent.name}")
+            self.tracker.track_agent_complete(agent.name, cached_decision)
+            return cached_decision  # type: ignore[no-any-return]
+
         confidence = await agent.can_handle(issue)
         self.tracker.track_agent_processing(agent.name, issue, confidence)
 
@@ -194,7 +293,8 @@ class AgentCoordinator:
         )
 
         try:
-            result = await agent.analyze_and_fix(issue)
+            # Use cached analysis for better performance
+            result = await self._cached_analyze_and_fix(agent, issue)
             if result.success:
                 self.logger.info(f"{agent.name} successfully fixed issue")
             else:
@@ -226,45 +326,6 @@ class AgentCoordinator:
             self.tracker.track_agent_complete(agent.name, error_result)
             return error_result
 
-    async def _handle_with_collaboration(
-        self,
-        agent_scores: list[tuple[SubAgent, float]],
-        issue: Issue,
-    ) -> FixResult:
-        self.logger.info(
-            f"Using collaborative approach for issue: {issue.message[:100]}",
-        )
-
-        results: list[FixResult] = []
-
-        for agent, _ in agent_scores:
-            try:
-                result = await agent.analyze_and_fix(issue)
-                results.append(result)
-
-                if result.success and result.confidence >= 0.8:
-                    self.logger.info(f"{agent.name} solved issue with high confidence")
-                    break
-
-            except Exception as e:
-                self.logger.exception(f"Collaborative agent {agent.name} failed: {e}")
-                results.append(
-                    FixResult(
-                        success=False,
-                        confidence=0.0,
-                        remaining_issues=[f"{agent.name} failed: {e}"],
-                    ),
-                )
-
-        if not results:
-            return FixResult(success=False, confidence=0.0)
-
-        combined_result = results[0]
-        for result in results[1:]:
-            combined_result = combined_result.merge_with(result)
-
-        return combined_result
-
     def _group_issues_by_type(
         self,
         issues: list[Issue],
@@ -274,6 +335,49 @@ class AgentCoordinator:
             grouped[issue.type].append(issue)
         return dict(grouped)
 
+    def _create_issue_hash(self, issue: Issue) -> str:
+        """Create a hash from issue content for caching decisions."""
+        content = (
+            f"{issue.type.value}:{issue.message}:{issue.file_path}:{issue.line_number}"
+        )
+        return hashlib.md5(content.encode(), usedforsecurity=False).hexdigest()
+
+    def _get_cache_key(self, agent_name: str, issue: Issue) -> str:
+        """Get cache key for agent-issue combination."""
+        issue_hash = self._create_issue_hash(issue)
+        return f"{agent_name}:{issue_hash}"
+
+    async def _cached_analyze_and_fix(self, agent: SubAgent, issue: Issue) -> FixResult:
+        """Analyze and fix issue with intelligent caching."""
+        cache_key = self._get_cache_key(agent.name, issue)
+
+        # Check in-memory cache first (fastest)
+        if cache_key in self._issue_cache:
+            self.logger.debug(f"Using in-memory cache for {agent.name}")
+            return self._issue_cache[cache_key]
+
+        # Check persistent cache
+        cached_result = self.cache.get_agent_decision(
+            agent.name, self._create_issue_hash(issue)
+        )
+        if cached_result:
+            self.logger.debug(f"Using persistent cache for {agent.name}")
+            # Store in memory cache for even faster future access
+            self._issue_cache[cache_key] = cached_result
+            return cached_result  # type: ignore[no-any-return]
+
+        # No cache hit - perform actual analysis
+        result = await agent.analyze_and_fix(issue)
+
+        # Cache successful results with high confidence
+        if result.success and result.confidence > 0.7:
+            self._issue_cache[cache_key] = result
+            self.cache.set_agent_decision(
+                agent.name, self._create_issue_hash(issue), result
+            )
+
+        return result
+
     def get_agent_capabilities(self) -> dict[str, dict[str, t.Any]]:
         if not self.agents:
             self.initialize_agents()
@@ -286,34 +390,7 @@ class AgentCoordinator:
             }
         return capabilities
 
-    def clear_cache(self) -> None:
-        self._issue_cache.clear()
-        self.logger.info("Cleared issue cache")
-
-    async def test_agent_connectivity(self) -> dict[str, bool]:
-        if not self.agents:
-            self.initialize_agents()
-
-        test_issue = Issue(
-            id="test",
-            type=IssueType.FORMATTING,
-            severity=Priority.LOW,
-            message="Test connectivity",
-        )
-
-        connectivity = {}
-        for agent in self.agents:
-            try:
-                confidence = await agent.can_handle(test_issue)
-                connectivity[agent.name] = confidence >= 0.0
-            except Exception as e:
-                self.logger.exception(f"Connectivity test failed for {agent.name}: {e}")
-                connectivity[agent.name] = False
-
-        return connectivity
-
     async def handle_issues_proactively(self, issues: list[Issue]) -> FixResult:
-        """Handle issues with proactive planning phase."""
         if not self.proactive_mode:
             return await self.handle_issues(issues)
 
@@ -325,13 +402,10 @@ class AgentCoordinator:
 
         self.logger.info(f"Handling {len(issues)} issues with proactive planning")
 
-        # Phase 1: Create architectural plan
         architectural_plan = await self._create_architectural_plan(issues)
 
-        # Phase 2: Apply fixes following the plan
         overall_result = await self._apply_fixes_with_plan(issues, architectural_plan)
 
-        # Phase 3: Validate against plan
         validation_result = await self._validate_against_plan(
             overall_result, architectural_plan
         )
@@ -339,78 +413,111 @@ class AgentCoordinator:
         return validation_result
 
     async def _create_architectural_plan(self, issues: list[Issue]) -> dict[str, t.Any]:
-        """Create architectural plan using ArchitectAgent."""
         architect = self._get_architect_agent()
 
         if not architect:
-            self.logger.warning("No ArchitectAgent available for planning")
-            return {"strategy": "reactive_fallback", "patterns": []}
-
-        # Analyze all issues together for comprehensive planning
-        complex_issues = [
-            issue
-            for issue in issues
-            if issue.type
-            in {IssueType.COMPLEXITY, IssueType.DRY_VIOLATION, IssueType.PERFORMANCE}
-        ]
+            return self._create_fallback_plan(
+                "No ArchitectAgent available for planning"
+            )
 
+        complex_issues = self._filter_complex_issues(issues)
         if not complex_issues:
             return {"strategy": "simple_fixes", "patterns": ["standard_patterns"]}
 
-        # Use the first complex issue for planning (represents the architectural challenge)
+        return await self._generate_architectural_plan(
+            architect, complex_issues, issues
+        )
+
+    def _create_fallback_plan(self, reason: str) -> dict[str, t.Any]:
+        """Create a fallback plan when architectural planning fails."""
+        self.logger.warning(reason)
+        return {"strategy": "reactive_fallback", "patterns": []}
+
+    def _filter_complex_issues(self, issues: list[Issue]) -> list[Issue]:
+        """Filter issues that require architectural planning."""
+        complex_types = {
+            IssueType.COMPLEXITY,
+            IssueType.DRY_VIOLATION,
+            IssueType.PERFORMANCE,
+        }
+        return [issue for issue in issues if issue.type in complex_types]
+
+    async def _generate_architectural_plan(
+        self, architect: t.Any, complex_issues: list[Issue], all_issues: list[Issue]
+    ) -> dict[str, t.Any]:
+        """Generate architectural plan using the architect agent."""
         primary_issue = complex_issues[0]
 
         try:
-            # Get plan from ArchitectAgent
             plan = await architect.plan_before_action(primary_issue)
-
-            # Extend plan to cover all issues
-            plan["all_issues"] = [issue.id for issue in issues]
-            plan["issue_types"] = list({issue.type.value for issue in issues})
+            plan = self._enrich_architectural_plan(plan, all_issues)
 
             self.logger.info(
                 f"Created architectural plan: {plan.get('strategy', 'unknown')}"
             )
-            return plan
+            return plan  # type: ignore[no-any-return]
 
         except Exception as e:
             self.logger.exception(f"Failed to create architectural plan: {e}")
             return {"strategy": "reactive_fallback", "patterns": [], "error": str(e)}
 
+    def _enrich_architectural_plan(
+        self, plan: dict[str, t.Any], issues: list[Issue]
+    ) -> dict[str, t.Any]:
+        """Enrich the architectural plan with issue metadata."""
+        plan["all_issues"] = [issue.id for issue in issues]
+        plan["issue_types"] = list[t.Any]({issue.type.value for issue in issues})
+        return plan
+
     async def _apply_fixes_with_plan(
         self, issues: list[Issue], plan: dict[str, t.Any]
     ) -> FixResult:
-        """Apply fixes following the architectural plan."""
         strategy = plan.get("strategy", "reactive_fallback")
 
         if strategy == "reactive_fallback":
-            # Fallback to standard processing
             return await self.handle_issues(issues)
 
         self.logger.info(f"Applying fixes with {strategy} strategy")
-
-        # Group issues by priority based on plan
         prioritized_issues = self._prioritize_issues_by_plan(issues, plan)
 
+        return await self._process_prioritized_groups(prioritized_issues, plan)
+
+    async def _process_prioritized_groups(
+        self, prioritized_issues: list[list[Issue]], plan: dict[str, t.Any]
+    ) -> FixResult:
+        """Process prioritized issue groups according to the plan."""
         overall_result = FixResult(success=True, confidence=1.0)
 
         for issue_group in prioritized_issues:
             group_result = await self._handle_issue_group_with_plan(issue_group, plan)
             overall_result = overall_result.merge_with(group_result)
 
-            # If a critical group fails, consider the overall strategy
-            if not group_result.success and self._is_critical_group(issue_group, plan):
-                overall_result.success = False
-                overall_result.remaining_issues.append(
-                    f"Critical issue group failed: {[i.id for i in issue_group]}"
+            if self._should_fail_on_group_failure(group_result, issue_group, plan):
+                overall_result = self._mark_critical_group_failure(
+                    overall_result, issue_group
                 )
 
         return overall_result
 
+    def _should_fail_on_group_failure(
+        self, group_result: FixResult, issue_group: list[Issue], plan: dict[str, t.Any]
+    ) -> bool:
+        """Determine if overall process should fail when a group fails."""
+        return not group_result.success and self._is_critical_group(issue_group, plan)
+
+    def _mark_critical_group_failure(
+        self, overall_result: FixResult, issue_group: list[Issue]
+    ) -> FixResult:
+        """Mark overall result as failed due to critical group failure."""
+        overall_result.success = False
+        overall_result.remaining_issues.append(
+            f"Critical issue group failed: {[i.id for i in issue_group]}"
+        )
+        return overall_result
+
     async def _validate_against_plan(
         self, result: FixResult, plan: dict[str, t.Any]
     ) -> FixResult:
-        """Validate the results against the architectural plan."""
         validation_steps = plan.get("validation", [])
 
         if not validation_steps:
@@ -418,7 +525,6 @@ class AgentCoordinator:
 
         self.logger.info(f"Validating against plan: {validation_steps}")
 
-        # Add validation recommendations
         result.recommendations.extend(
             [
                 f"Validate with: {', '.join(validation_steps)}",
@@ -430,7 +536,6 @@ class AgentCoordinator:
         return result
 
     def _get_architect_agent(self) -> SubAgent | None:
-        """Get the ArchitectAgent from available agents."""
         for agent in self.agents:
             if agent.__class__.__name__ == "ArchitectAgent":
                 return agent
@@ -439,11 +544,9 @@ class AgentCoordinator:
     def _prioritize_issues_by_plan(
         self, issues: list[Issue], plan: dict[str, t.Any]
     ) -> list[list[Issue]]:
-        """Prioritize issues based on the architectural plan."""
         strategy = plan.get("strategy", "reactive_fallback")
 
         if strategy == "external_specialist_guided":
-            # Handle complex issues first, then simple ones
             complex_issues = [
                 issue
                 for issue in issues
@@ -452,24 +555,20 @@ class AgentCoordinator:
             other_issues = [issue for issue in issues if issue not in complex_issues]
             return [complex_issues, other_issues] if complex_issues else [other_issues]
 
-        # Default: group by type for parallel processing
         groups = self._group_issues_by_type(issues)
-        return list(groups.values())
+        return list[t.Any](groups.values())
 
     async def _handle_issue_group_with_plan(
         self, issues: list[Issue], plan: dict[str, t.Any]
     ) -> FixResult:
-        """Handle a group of issues with architectural guidance."""
         if not issues:
             return FixResult(success=True, confidence=1.0)
 
-        # Get the best agent for this group, preferring ArchitectAgent for complex issues
         representative_issue = issues[0]
 
         if self._should_use_architect_for_group(issues, plan):
             architect = self._get_architect_agent()
             if architect:
-                # Use architect for the whole group
                 group_result = FixResult(success=True, confidence=1.0)
 
                 for issue in issues:
@@ -478,20 +577,16 @@ class AgentCoordinator:
 
                 return group_result
 
-        # Use standard routing for simpler issues
         return await self._handle_issues_by_type(representative_issue.type, issues)
 
     def _should_use_architect_for_group(
         self, issues: list[Issue], plan: dict[str, t.Any]
     ) -> bool:
-        """Determine if ArchitectAgent should handle this group."""
         strategy = plan.get("strategy", "")
 
-        # Always use architect for specialist-guided strategies
         if strategy == "external_specialist_guided":
             return True
 
-        # Use architect for complex or architectural issues
         architectural_types = {
             IssueType.COMPLEXITY,
             IssueType.DRY_VIOLATION,
@@ -501,12 +596,9 @@ class AgentCoordinator:
         return any(issue.type in architectural_types for issue in issues)
 
     def _is_critical_group(self, issues: list[Issue], plan: dict[str, t.Any]) -> bool:
-        """Determine if this issue group is critical to the plan."""
-        # Complex and DRY violations are typically critical
         critical_types = {IssueType.COMPLEXITY, IssueType.DRY_VIOLATION}
         return any(issue.type in critical_types for issue in issues)
 
     def set_proactive_mode(self, enabled: bool) -> None:
-        """Enable or disable proactive planning mode."""
         self.proactive_mode = enabled
         self.logger.info(f"Proactive mode {'enabled' if enabled else 'disabled'}")
